@@ -19,7 +19,6 @@ from lxml import etree  # pytype: disable=import-error
 import re
 from typing import List, Optional, Tuple
 from picosvg.svg_meta import ntos, svgns, xlinkns
-from picosvg import svg_pathops
 from picosvg.svg_types import *
 import numbers
 
@@ -205,13 +204,13 @@ class SVG:
             self.elements[idx] = (el, (shape.as_path(),))
         return self
 
-    def _xpath(self, xpath, el=None):
+    def xpath(self, xpath, el=None):
         if el is None:
             el = self.svg_root
         return el.xpath(xpath, namespaces={"svg": svgns()})
 
-    def _xpath_one(self, xpath):
-        els = self._xpath(xpath)
+    def xpath_one(self, xpath):
+        els = self.xpath(xpath)
         if len(els) != 1:
             raise ValueError(f"Need exactly 1 match for {xpath}, got {len(els)}")
         return els[0]
@@ -220,20 +219,21 @@ class SVG:
         match = re.match(r"^url[(]#([\w-]+)[)]$", url)
         if not match:
             raise ValueError(f'Unrecognized url "{url}"')
-        return self._xpath_one(f'//svg:{el_tag}[@id="{match.group(1)}"]')
+        return self.xpath_one(f'//svg:{el_tag}[@id="{match.group(1)}"]')
 
     def _resolve_use(self, scope_el):
         attrib_not_copied = {"x", "y", "width", "height", _xlink_href_attr_name()}
 
         swaps = []
 
-        for use_el in self._xpath(".//svg:use", el=scope_el):
+        for use_el in self.xpath(".//svg:use", el=scope_el):
             ref = use_el.attrib.get(_xlink_href_attr_name(), "")
             if not ref.startswith("#"):
                 raise ValueError("Only use #fragment supported")
-            target = self._xpath_one(f'//svg:*[@id="{ref[1:]}"]')
+            target = self.xpath_one(f'//svg:*[@id="{ref[1:]}"]')
 
             new_el = copy.deepcopy(target)
+            del new_el.attrib["id"]
 
             group = etree.Element(f"{{{svgns()}}}g", nsmap=self.svg_root.nsmap)
             use_x = use_el.attrib.get("x", 0)
@@ -277,10 +277,12 @@ class SVG:
 
         # union all the shapes under the clipPath
         # Fails if there are any non-shapes under clipPath
-        clip_path = SVGPath.from_commands(
-            svg_pathops.union(*[from_element(e).as_cmd_seq() for e in clip_path_el])
-        )
-        return clip_path
+        return union([from_element(e) for e in clip_path_el])
+
+    def append_to(self, xpath, el):
+        self._update_etree()
+        self.xpath_one(xpath).append(el)
+        return el
 
     def _combine_clip_paths(self, clip_paths) -> SVGPath:
         # multiple clip paths leave behind their intersection
@@ -288,14 +290,12 @@ class SVG:
             raise ValueError("Cannot combine no clip_paths")
         if len(clip_paths) == 1:
             return clip_paths[0]
-        return SVGPath.from_commands(
-            svg_pathops.intersection(*[c.as_cmd_seq() for c in clip_paths])
-        )
+        return intersection([c.as_cmd_seq() for c in clip_paths])
 
     def _new_id(self, tag, template):
         for i in range(100):
             potential_id = template % i
-            existing = self._xpath(f'//svg:{tag}[@id="{potential_id}"]')
+            existing = self.xpath(f'//svg:{tag}[@id="{potential_id}"]')
             if not existing:
                 return potential_id
         raise ValueError(f"No free id for {template}")
@@ -354,13 +354,13 @@ class SVG:
         If result has multiple clip paths merge them.
         """
         # nuke the groups that are not displayed
-        display_none = [e for e in self._xpath(f".//svg:g[@display='none']", scope_el)]
+        display_none = [e for e in self.xpath(f".//svg:g[@display='none']", scope_el)]
         for group in display_none:
             if group.getparent() is not None:
                 group.getparent().remove(group)
 
         # Any groups left are displayed
-        groups = [e for e in self._xpath(f".//svg:g", scope_el)]
+        groups = [e for e in self.xpath(f".//svg:g", scope_el)]
         multi_clips = []
         for group in groups:
             # move groups children up
@@ -405,7 +405,7 @@ class SVG:
             if old_clip_path.getparent() is None:
                 continue
             old_id = old_clip_path.attrib["id"]
-            if not self._xpath(f'//svg:*[@clip-path="url(#{old_id})"]'):
+            if not self.xpath(f'//svg:*[@clip-path="url(#{old_id})"]'):
                 old_clip_path.getparent().remove(old_clip_path)
 
     def _compute_clip_path(self, el):
@@ -442,38 +442,21 @@ class SVG:
         Returns sequence of shapes in draw order. That is, result[1] should be
         drawn on top of result[0], etc."""
 
-        def stroke_pred(field):
-            return field.name.startswith("stroke")
-
-        # map old fields to new dest
-        _stroke_fields = {"stroke": "fill", "stroke_opacity": "opacity"}
-
         if shape.stroke == "none":
             return (shape,)
 
         # make a new path that is the stroke
-        stroke = SVGPath.from_commands(
-            svg_pathops.stroke(
-                shape.as_cmd_seq(),
-                shape.stroke_linecap,
-                shape.stroke_linejoin,
-                shape.stroke_width,
-                shape.stroke_miterlimit,
-                self.tolerance,
-            )
-        )
+        stroke = shape.as_path().update_path(shape.stroke_commands(self.tolerance))
 
-        # convert some stroke attrs (e.g. stroke => fill)
-        for field in dataclasses.fields(shape):
-            dest_field = _stroke_fields.get(field.name, None)
-            if not dest_field:
-                continue
-            setattr(stroke, dest_field, getattr(shape, field.name))
+        # a few attributes move in interesting ways
+        stroke.opacity *= stroke.stroke_opacity
+        stroke.fill = stroke.stroke
 
         # remove all the stroke settings
-        _reset_attrs(shape, stroke_pred)
+        for cleanmeup in (shape, stroke):
+            _reset_attrs(cleanmeup, lambda field: field.name.startswith("stroke"))
 
-        if shape.fill == "none":
+        if not shape.visible():
             return (stroke,)
 
         return (shape, stroke)
@@ -526,16 +509,17 @@ class SVG:
         # apply clip path to target
         for el_idx, clip_path in clips:
             el, (target,) = self.elements[el_idx]
-            target = target.as_path().absolute(inplace=True)
-
-            target.d = SVGPath.from_commands(
-                svg_pathops.intersection(target.as_cmd_seq(), clip_path.as_cmd_seq())
-            ).d
+            target = (
+                target.as_path()
+                .absolute(inplace=True)
+                .update_path(intersection((target, clip_path)), inplace=True,)
+            )
             target.clip_path = ""
+
             self._set_element(el_idx, el, (target,))
 
         # destroy clip path elements
-        for clip_path_el in self._xpath("//svg:clipPath"):
+        for clip_path_el in self.xpath("//svg:clipPath"):
             clip_path_el.getparent().remove(clip_path_el)
 
         # destroy clip-path attributes
@@ -605,7 +589,7 @@ class SVG:
 
         self._update_etree()
 
-        for el in self._xpath("//comment()"):
+        for el in self.xpath("//comment()"):
             el.getparent().remove(el)
 
         return self
@@ -618,7 +602,7 @@ class SVG:
 
         self._update_etree()
 
-        for el in self._xpath(xpath):
+        for el in self.xpath(xpath):
             for name, value in name_values:
                 el.attrib[name] = value
 
@@ -633,7 +617,7 @@ class SVG:
 
         self._update_etree()
 
-        for el in self._xpath(xpath):
+        for el in self.xpath(xpath):
             _del_attrs(el, *names)
 
         return self
@@ -699,11 +683,11 @@ class SVG:
 
         # Collect gradients; remove other defs
         defs = etree.Element(f"{{{svgns()}}}defs", nsmap=self.svg_root.nsmap)
-        for gradient in self._xpath("//svg:linearGradient | //svg:radialGradient"):
+        for gradient in self.xpath("//svg:linearGradient | //svg:radialGradient"):
             gradient.getparent().remove(gradient)
             defs.append(gradient)
 
-        for def_el in [e for e in self._xpath("//svg:defs")]:
+        for def_el in [e for e in self.xpath("//svg:defs")]:
             def_el.getparent().remove(def_el)
 
         self.svg_root.insert(0, defs)
