@@ -20,7 +20,7 @@ from picosvg import svg_pathops
 from picosvg.arc_to_cubic import arc_to_cubic
 from picosvg.svg_path_iter import parse_svg_path
 from picosvg.svg_transform import Affine2D
-from typing import Generator
+from typing import Generator, Iterable
 
 
 # Subset of https://www.w3.org/TR/SVG11/painting.html
@@ -85,7 +85,10 @@ class SVGShape:
 
     def apply_transform(self, transform: Affine2D) -> "SVGPath":
         cmds = svg_pathops.transform(self.as_cmd_seq(), transform)
-        return SVGPath.from_commands(cmds)
+        target = self.as_path()
+        if target is self:
+            target = copy.deepcopy(target)
+        return target.update_path(cmds, inplace=True)
 
     def as_path(self) -> "SVGPath":
         raise NotImplementedError("You should implement as_path")
@@ -104,6 +107,16 @@ class SVGShape:
         # only meaningful for path, which overrides
         return self
 
+    def stroke_commands(self, tolerance) -> Generator[svg_meta.SVGCommand, None, None]:
+        return svg_pathops.stroke(
+            self.as_cmd_seq(),
+            self.stroke_linecap,
+            self.stroke_linejoin,
+            self.stroke_width,
+            self.stroke_miterlimit,
+            tolerance,
+        )
+
 
 # https://www.w3.org/TR/SVG11/paths.html#PathElement
 @dataclasses.dataclass
@@ -362,10 +375,19 @@ class SVGPath(SVGShape, svg_meta.SVGCommandSeq):
     def from_commands(
         cls, svg_cmds: Generator[svg_meta.SVGCommand, None, None]
     ) -> "SVGPath":
-        svg_path = cls()
+        return cls().update_path(svg_cmds, inplace=True)
+
+    def update_path(
+        self, svg_cmds: Generator[svg_meta.SVGCommand, None, None], inplace=False
+    ) -> "SVGPath":
+        target = self
+        if not inplace:
+            target = copy.deepcopy(self)
+        target.d = ""
+
         for cmd, args in svg_cmds:
-            svg_path._add_cmd(cmd, *args)
-        return svg_path
+            target._add_cmd(cmd, *args)
+        return target
 
 
 # https://www.w3.org/TR/SVG11/shapes.html#CircleElement
@@ -487,3 +509,13 @@ class SVGRect(SVGShape):
         path._copy_common_fields(*shape_fields)
 
         return path
+
+
+def union(shapes: Iterable[SVGShape]) -> SVGPath:
+    return SVGPath.from_commands(svg_pathops.union(*[s.as_cmd_seq() for s in shapes]))
+
+
+def intersection(shapes: Iterable[SVGShape]) -> SVGPath:
+    return SVGPath.from_commands(
+        svg_pathops.intersection(*[s.as_cmd_seq() for s in shapes])
+    )
